@@ -6,12 +6,12 @@ PDL::Basic -- Basic utility functions for PDL
 =head1 DESCRIPTION
 
 This module contains basic utility functions for
-creating and manipulating piddles. Most of these functions
+creating and manipulating ndarrays. Most of these functions
 are simplified interfaces to the more flexible functions in
-the modules 
-L<PDL::Primitive|PDL::Primitive> 
-and 
-L<PDL::Slices|PDL::Slices>.
+the modules
+L<PDL::Primitive>
+and
+L<PDL::Slices>.
 
 =head1 SYNOPSIS
 
@@ -22,16 +22,18 @@ L<PDL::Slices|PDL::Slices>.
 =cut
 
 package PDL::Basic;
+use strict;
+use warnings;
 use PDL::Core '';
 use PDL::Types;
 use PDL::Exporter;
 use PDL::Options;
 
-@ISA=qw/PDL::Exporter/;
-@EXPORT_OK = qw/ ndcoords rvals axisvals allaxisvals xvals yvals zvals sec ins hist whist
+our @ISA=qw/PDL::Exporter/;
+our @EXPORT_OK = qw/ ndcoords rvals axisvals allaxisvals xvals yvals zvals sec ins hist whist
 	similar_assign transpose sequence xlinvals ylinvals
 	zlinvals axislinvals/;
-%EXPORT_TAGS = (Func=>[@EXPORT_OK]);
+our %EXPORT_TAGS = (Func=>[@EXPORT_OK]);
 
 # Exportable functions
 *axisvals       = \&PDL::axisvals;		
@@ -50,19 +52,20 @@ use PDL::Options;
 
 =for ref
 
-Fills a piddle with X index values.  Uses similar specifications to
-L<zeroes|zeroes> and L<new_from_specification|new_from_specification>.
+Fills an ndarray with X index values.  Uses similar specifications to
+L</zeroes> and L</new_from_specification>.
 
-CAVEAT: 
+CAVEAT:
 
-If you use the single argument piddle form (top row
-in the usage table) the output will have the same type as the input;
-this may give surprising results if, e.g., you have a byte array with
-a dimension of size greater than 256.  To force a type, use the third form.
+If you use the single argument ndarray form (top row
+in the usage table) the output will have the same type as the input,
+except that as of 2.064, the returned ndarray will default to at least type
+C<double>. As of 2.085, this will respect a given type as in the second
+or third form below.
 
 =for usage
 
- $x = xvals($somearray);
+ $x = xvals($somearray); # at least type double
  $x = xvals([OPTIONAL TYPE],$nx,$ny,$nz...);
  $x = xvals([OPTIONAL TYPE], $somarray->dims);
 
@@ -88,7 +91,7 @@ etc. see L<zeroes|PDL::Core/zeroes>.
 
 =for ref
 
-Fills a piddle with Y index values.  See the CAVEAT for L<xvals|xvals>.
+Fills an ndarray with Y index values.  See the CAVEAT for L</xvals>.
 
 =for usage
 
@@ -117,7 +120,7 @@ etc. see L<zeroes|PDL::Core/zeroes>.
 
 =for ref
 
-Fills a piddle with Z index values.  See the CAVEAT for L<xvals|xvals>.
+Fills an ndarray with Z index values.  See the CAVEAT for L</xvals>.
 
 =for usage
 
@@ -148,18 +151,18 @@ etc. see L<zeroes|PDL::Core/zeroes>.
 
 =for ref
 
-X axis values between endpoints (see L<xvals|/xvals>).
+X axis values between endpoints (see L</xvals>).
 
 =for usage
 
- $a = zeroes(100,100);
- $x = $a->xlinvals(0.5,1.5);
- $y = $a->ylinvals(-2,-1);
+ $w = zeroes(100,100);
+ $x = $w->xlinvals(0.5,1.5);
+ $y = $w->ylinvals(-2,-1);
  # calculate Z for X between 0.5 and 1.5 and
  # Y between -2 and -1.
- $z = f($x,$y);            
+ $z = f($x,$y);
 
-C<xlinvals>, C<ylinvals> and C<zlinvals> return a piddle with the same shape
+C<xlinvals>, C<ylinvals> and C<zlinvals> return an ndarray with the same shape
 as their first argument and linearly scaled values between the two other
 arguments along the given axis.
 
@@ -167,52 +170,52 @@ arguments along the given axis.
 
 =for ref
 
-Y axis values between endpoints (see L<yvals|/yvals>).
+Y axis values between endpoints (see L</yvals>).
 
-See L<xlinvals|/xlinvals> for more information.
+See L</xlinvals> for more information.
 
 =head2 zlinvals
 
 =for ref
 
-Z axis values between endpoints (see L<zvals|/zvals>).
+Z axis values between endpoints (see L</zvals>).
 
-See L<xlinvals|/xlinvals> for more information.
+See L</xlinvals> for more information.
 
 =head2 xlogvals
 
 =for ref
 
-X axis values logarithmicly spaced between endpoints (see L<xvals|/xvals>).
+X axis values logarithmically spaced between endpoints (see L</xvals>).
 
 =for usage
 
- $a = zeroes(100,100);
- $x = $a->xlogvals(1e-6,1e-3);
- $y = $a->ylinvals(1e-4,1e3);
+ $w = zeroes(100,100);
+ $x = $w->xlogvals(1e-6,1e-3);
+ $y = $w->ylinvals(1e-4,1e3);
  # calculate Z for X between 1e-6 and 1e-3 and
  # Y between 1e-4 and 1e3.
- $z = f($x,$y);            
+ $z = f($x,$y);
 
-C<xlogvals>, C<ylogvals> and C<zlogvals> return a piddle with the same shape
-as their first argument and logarithmicly scaled values between the two other
+C<xlogvals>, C<ylogvals> and C<zlogvals> return an ndarray with the same shape
+as their first argument and logarithmically scaled values between the two other
 arguments along the given axis.
 
 =head2 ylogvals
 
 =for ref
 
-Y axis values logarithmicly spaced between endpoints (see L<yvals|/yvals>).
+Y axis values logarithmically spaced between endpoints (see L</yvals>).
 
-See L<xlogvals|/xlogvals> for more information.
+See L</xlogvals> for more information.
 
 =head2 zlogvals
 
 =for ref
 
-Z axis values logarithmicly spaced between endpoints (see L<zvals|/zvals>).
+Z axis values logarithmically spaced between endpoints (see L</zvals>).
 
-See L<xlogvals|/xlogvals> for more information.
+See L</xlogvals> for more information.
 
 =cut
 
@@ -222,93 +225,68 @@ sub xvals { ref($_[0]) && ref($_[0]) ne 'PDL::Type' ? $_[0]->xvals : PDL->xvals(
 sub yvals { ref($_[0]) && ref($_[0]) ne 'PDL::Type' ? $_[0]->yvals : PDL->yvals(@_) }
 sub zvals { ref($_[0]) && ref($_[0]) ne 'PDL::Type' ? $_[0]->zvals : PDL->zvals(@_) }
 sub PDL::xvals {
-    my $class = shift;
-    my $pdl = scalar(@_)? $class->new_from_specification(@_) : $class->new_or_inplace;
-    axisvals2($pdl,0);
-    return $pdl;
+    my $type_given = grep +(ref($_[$_])||'') eq 'PDL::Type', 0..1;
+    axisvals2(&PDL::Core::_construct,0,$type_given);
 }
 sub PDL::yvals {
-    my $class = shift;
-    my $pdl = scalar(@_)? $class->new_from_specification(@_) : $class->new_or_inplace;
-    axisvals2($pdl,1);
-    return $pdl;
+    my $type_given = grep +(ref($_[$_])||'') eq 'PDL::Type', 0..1;
+    axisvals2(&PDL::Core::_construct,1,$type_given);
 }
 sub PDL::zvals {
-    my $class = shift;
-    my $pdl = scalar(@_)? $class->new_from_specification(@_) : $class->new_or_inplace;
-    axisvals2($pdl,2);
-    return $pdl;
+    my $type_given = grep +(ref($_[$_])||'') eq 'PDL::Type', 0..1;
+    axisvals2(&PDL::Core::_construct,2,$type_given);
 }
 
+sub _dimcheck {
+  my ($pdl, $whichdim, $name) = @_;
+  my $dim = $pdl->getdim($whichdim);
+  barf "Must have at least two elements in dimension for $name" if $dim <= 1;
+  $dim;
+}
+sub _linvals {
+  my ($pdl, $v1, $v2, $dim, $method) = @_;
+  $pdl->$method * (($v2 - $v1) / ($dim-1)) + $v1;
+}
 sub PDL::xlinvals {
-	my $dim = $_[0]->getdim(0);
-	barf "Must have at least two elements in dimension for xlinvals"
-		if $dim <= 1;
-	return $_[0]->xvals * (($_[2] - $_[1]) / ($dim-1)) + $_[1];
+  _linvals(@_[0..2], _dimcheck($_[0], 0, 'xlinvals'), 'xvals');
 }
-
 sub PDL::ylinvals {
-	my $dim = $_[0]->getdim(1);
-	barf "Must have at least two elements in dimension for ylinvals"
-		if $dim <= 1;
-	return $_[0]->yvals * (($_[2] - $_[1]) / ($dim-1)) + $_[1];
+  _linvals(@_[0..2], _dimcheck($_[0], 1, 'ylinvals'), 'yvals');
 }
-
 sub PDL::zlinvals {
-	my $dim = $_[0]->getdim(2);
-	barf "Must have at least two elements in dimension for zlinvals"
-		if $dim <= 1;
-	return $_[0]->zvals * (($_[2] - $_[1]) / ($dim-1)) + $_[1];
+  _linvals(@_[0..2], _dimcheck($_[0], 2, 'zlinvals'), 'zvals');
 }
 
+sub _logvals {
+  my ($pdl, $min, $max, $dim, $method) = @_;
+  barf "min and max must be positive" if $min <= 0 || $max <= 0;
+  my ($lmin,$lmax) = map log($_), $min, $max;
+  exp($pdl->$method * (($lmax - $lmin) / ($dim-1)) + $lmin);
+}
 sub PDL::xlogvals {
-	my $dim = $_[0]->getdim(0);
-	barf "Must have at least two elements in dimension for xlogvals"
-		if $dim <= 1;
-	my ($xmin,$xmax) = @_[1,2];
-	barf "xmin and xmax must be positive"
-	  if $xmin <= 0 || $xmax <= 0;
-	my ($lxmin,$lxmax) = (log($xmin), log($xmax));
-	return exp($_[0]->xvals * (($lxmax - $lxmin) / ($dim-1)) + $lxmin);
+  _logvals(@_[0..2], _dimcheck($_[0], 0, 'xlogvals'), 'xvals');
 }
-
 sub PDL::ylogvals {
-	my $dim = $_[0]->getdim(1);
-	barf "Must have at least two elements in dimension for xlogvals"
-		if $dim <= 1;
-	my ($xmin,$xmax) = @_[1,2];
-	barf "xmin and xmax must be positive"
-	  if $xmin <= 0 || $xmax <= 0;
-	my ($lxmin,$lxmax) = (log($xmin), log($xmax));
-	return exp($_[0]->yvals * (($lxmax - $lxmin) / ($dim-1)) + $lxmin);
+  _logvals(@_[0..2], _dimcheck($_[0], 1, 'ylogvals'), 'yvals');
 }
-
 sub PDL::zlogvals {
-	my $dim = $_[0]->getdim(2);
-	barf "Must have at least two elements in dimension for xlogvals"
-		if $dim <= 1;
-	my ($xmin,$xmax) = @_[1,2];
-	barf "xmin and xmax must be positive"
-	  if $xmin <= 0 || $xmax <= 0;
-	my ($lxmin,$lxmax) = (log($xmin), log($xmax));
-	return exp($_[0]->zvals * (($lxmax - $lxmin) / ($dim-1)) + $lxmin);
+  _logvals(@_[0..2], _dimcheck($_[0], 2, 'zlogvals'), 'zvals');
 }
 
-
 =head2 allaxisvals
 
 =for ref
 
-Synonym for L<ndcoords|ndcoords> - enumerates all coordinates in a
-PDL or dim list, adding an extra dim on the front to accomodate
-the vector coordinate index (the form expected by L<indexND|indexND>,
-L<range|range>, and L<interpND|interpND>).  See L<ndcoords|ndcoords> for more detail.
+Synonym for L</ndcoords> - enumerates all coordinates in a
+PDL or dim list, adding an extra dim on the front to accommodate
+the vector coordinate index (the form expected by L</indexND>,
+L</range>, and L</interpND>).  See L</ndcoords> for more detail.
 
 =for usage
 
-$indices = allaxisvals($pdl);
-$indices = allaxisvals(@dimlist);
-$indices = allaxisvals($type,@dimlist);
+  $indices = allaxisvals($pdl);
+  $indices = allaxisvals(@dimlist);
+  $indices = allaxisvals($type,@dimlist);
 
 =cut
 
@@ -316,25 +294,25 @@ $indices = allaxisvals($type,@dimlist);
 
 =for ref
 
-Enumerate pixel coordinates for an N-D piddle
+Enumerate pixel coordinates for an N-D ndarray
 
 Returns an enumerated list of coordinates suitable for use in
 L<indexND|PDL::Slices/indexND> or L<range|PDL::Slices/range>: you feed
-in a dimension list and get out a piddle whose 0th dimension runs over
+in a dimension list and get out an ndarray whose 0th dimension runs over
 dimension index and whose 1st through Nth dimensions are the
-dimensions given in the input.  If you feed in a piddle instead of a
-perl list, then the dimension list is used, as in L<xvals|xvals> etc.
+dimensions given in the input.  If you feed in an ndarray instead of a
+perl list, then the dimension list is used, as in L</xvals> etc.
 
-Unlike L<xvals|xvals> etc., if you supply a piddle input, you get 
-out a piddle of the default piddle type: double.   This causes less
+Unlike L</xvals> etc., if you supply an ndarray input, you get
+out an ndarray of the default ndarray type: double.   This causes less
 surprises than the previous default of keeping the data type of
-the input piddle since that rarely made sense in most usages.
+the input ndarray since that rarely made sense in most usages.
 
 =for usage
 
-$indices = ndcoords($pdl);
-$indices = ndcoords(@dimlist);
-$indices = ndcoords($type,@dimlist);
+  $indices = ndcoords($pdl);
+  $indices = ndcoords(@dimlist);
+  $indices = ndcoords($type,@dimlist);
 
 =for example
 
@@ -355,47 +333,41 @@ $indices = ndcoords($type,@dimlist);
    ]
   ]
 
-  pdl> $a = zeroes(byte,2,3);        # $a is a 2x3 byte piddle
-  pdl> $b = ndcoords($a);            # $b inherits $a's type
-  pdl> $c = ndcoords(long,$a->dims); # $c is a long piddle, same dims as $b
-  pdl> help $b;
+  pdl> $w = zeroes(byte,2,3);        # $w is a 2x3 byte ndarray
+  pdl> $y = ndcoords($w);            # $y inherits $w's type
+  pdl> $c = ndcoords(long,$w->dims); # $c is a long ndarray, same dims as $y
+  pdl> help $y;
   This variable is   Byte D [2,2,3]              P            0.01Kb
   pdl> help $c;
   This variable is   Long D [2,2,3]              P            0.05Kb
 
-
 =cut
 
-sub PDL::ndcoords { 
+sub PDL::ndcoords {
   my $type;
   if(ref $_[0] eq 'PDL::Type') {
     $type = shift;
   }
-  
   my @dims = (ref $_[0]) ? (shift)->dims : @_;
   my @d = @dims;
   unshift(@d,scalar(@dims));
   unshift(@d,$type) if defined($type);
-
-  $out = PDL->zeroes(@d);
-  
+  my $out = PDL->zeroes(@d);
   for my $d(0..$#dims) {
-    my $a = $out->index($d)->mv($d,0);
-    $a .= xvals($a);
+    my $w = $out->index($d)->mv($d,0);
+    $w .= xvals($w);
   }
-
   $out;
 }
 *ndcoords = \&PDL::ndcoords;
 *allaxisvals = \&PDL::ndcoords;
 *PDL::allaxisvals = \&PDL::ndcoords;
- 
 
 =head2 hist
 
 =for ref
 
-Create histogram of a piddle
+Create histogram of an ndarray
 
 =for usage
 
@@ -407,10 +379,11 @@ or
  $hist = hist($data,$min,$max,$step);
  ($xvals,$hist) = hist($data,[$min,$max,$step]);
 
-If C<hist> is run in list context, C<$xvals> gives the computed bin centres
+If C<hist> is run in list context, C<$xvals> gives the
+computed bin centres as double values.
 
-A nice idiom (with 
-L<PDL::Graphics::PGPLOT|PDL::Graphics::PGPLOT>) is
+A nice idiom (with
+L<PDL::Graphics::PGPLOT>) is
 
  bin hist $data;  # Plot histogram
 
@@ -425,22 +398,16 @@ L<PDL::Graphics::PGPLOT|PDL::Graphics::PGPLOT>) is
 =cut
 
 sub PDL::hist {
-
     my $usage = "\n" . '  Usage:          $hist  = hist($data)' . "\n" .
                        '                  $hist  = hist($data,$min,$max,$step)' . "\n" .
                        '          ($xvals,$hist) = hist($data)' . "\n" .
                        '          ($xvals,$hist) = hist($data,$min,$max,$step)' . "\n" ;
     barf($usage) if $#_<0;
-
     my($pdl,$min,$max,$step)=@_;
-    my $xvals;
-
-    ($step, $min, $bins, $xvals) = 
+    ($step, $min, my $bins, my $xvals) =
         _hist_bin_calc($pdl, $min, $max, $step, wantarray());
-
     PDL::Primitive::histogram($pdl->clump(-1),(my $hist = null),
 			      $step,$min,$bins);
-
     return wantarray() ? ($xvals,$hist) : $hist;
 }
 
@@ -448,18 +415,19 @@ sub PDL::hist {
 
 =for ref
 
-Create a weighted histogram of a piddle
+Create a weighted histogram of an ndarray
 
 =for usage
 
  $hist = whist($data, $wt, [$min,$max,$step]);
  ($xvals,$hist) = whist($data, $wt, [$min,$max,$step]);
 
-If requested, C<$xvals> gives the computed bin centres.
-C<$data> and C<$wt> should have the same dimensionality and extents.
+If requested, C<$xvals> gives the computed bin centres
+as type double values.  C<$data> and C<$wt> should have
+the same dimensionality and extents.
 
-A nice idiom (with 
-L<PDL::Graphics::PGPLOT|PDL::Graphics::PGPLOT>) is
+A nice idiom (with
+L<PDL::Graphics::PGPLOT>) is
 
  bin whist $data, $wt;  # Plot histogram
 
@@ -469,7 +437,7 @@ L<PDL::Graphics::PGPLOT|PDL::Graphics::PGPLOT>) is
  [13 10 13 10 9 13 9 12 11 10 10 13 7 6 8 10 11 7 12 9 11 11 12 6 12 7]
  pdl> $wt = grandom($y->nelem)
  pdl> $h = whist $y, $wt, 0, 20, 1 # hist with step 1, min 0 and 20 bins
- pdl> p $h                        
+ pdl> p $h
  [0 0 0 0 0 0 -0.49552342  1.7987439 0.39450696  4.0073722 -2.6255299 -2.5084501  2.6458365  4.1671676 0 0 0 0 0 0]
 
 
@@ -479,9 +447,7 @@ sub PDL::whist {
     barf('Usage: ([$xvals],$hist) = whist($data,$wt,[$min,$max,$step])')
             if @_ < 2;
     my($pdl,$wt,$min,$max,$step)=@_;
-    my $xvals;
-
-    ($step, $min, $bins, $xvals) = 
+    ($step, $min, my $bins, my $xvals) =
         _hist_bin_calc($pdl, $min, $max, $step, wantarray());
 
     PDL::Primitive::whistogram($pdl->clump(-1),$wt->clump(-1),
@@ -493,19 +459,17 @@ sub _hist_bin_calc {
     my($pdl,$min,$max,$step,$wantarray)=@_;
     $min = $pdl->min() unless defined $min;
     $max = $pdl->max() unless defined $max;
-    my $ntype = $pdl->get_datatype;
-    barf "empty piddle, no values to work with" if $pdl->nelem == 0;
-    unless (defined $step) {
-	my $defbins = 100 < $pdl->nelem ? 100 : $pdl->nelem;
-	$step = ($max-$min)/$defbins;
-	$step = int($step) > 0 ? int($step) : 1 if $ntype < $PDL_F;
-    }
+    my $nelem = $pdl->nelem;
+    barf "empty ndarray, no values to work with" if $nelem == 0;
+
+    $step = ($max-$min)/(($nelem>10_000) ? 100 : sqrt($nelem)) unless defined $step;
     barf "step is zero (or all data equal to one value)" if $step == 0;
-    my $bins = int(($max-$min)/$step);
+
+    my $bins = int(($max-$min)/$step+0.5);
     print "hist with step $step, min $min and $bins bins\n"
-      if $PDL::debug;
-    my $xvals = $min + $step/2 + sequence(PDL::Type->new($ntype),$bins)*
-        PDL::convert($step,$ntype) if $wantarray;
+	if $PDL::debug;
+    # Need to use double for $xvals here
+    my $xvals = $min + $step/2 + sequence(PDL::Core::double,$bins)*$step if $wantarray;
 
     return ( $step, $min, $bins, $xvals );
 }
@@ -519,7 +483,7 @@ Create array filled with a sequence of values
 
 =for usage
 
- $a = sequence($b); $a = sequence [OPTIONAL TYPE], @dims;
+ $w = sequence($y); $w = sequence [OPTIONAL TYPE], @dims;
 
 etc. see L<zeroes|PDL::Core/zeroes>.
 
@@ -539,10 +503,11 @@ etc. see L<zeroes|PDL::Core/zeroes>.
 
 sub sequence { ref($_[0]) && ref($_[0]) ne 'PDL::Type' ? $_[0]->sequence : PDL->sequence(@_) }
 sub PDL::sequence {
-    my $class = shift;
-    my $pdl = scalar(@_)? $class->new_from_specification(@_) : $class->new_or_inplace;
+    my $type_given = grep +(ref($_[$_])||'') eq 'PDL::Type', 0..1;
+    $type_given ||= ref($_[0]) && UNIVERSAL::isa($_[0], 'PDL'); # instance method
+    my $pdl = &PDL::Core::_construct;
     my $bar = $pdl->clump(-1)->inplace;
-    my $foo = $bar->xvals;
+    axisvals2($bar,0,$type_given);
     return $pdl;
 }
 
@@ -550,11 +515,11 @@ sub PDL::sequence {
 
 =for ref
 
-Fills a piddle with radial distance values from some centre.
+Fills an ndarray with radial distance values from some centre.
 
 =for usage
 
- $r = rvals $piddle,{OPTIONS};
+ $r = rvals $ndarray,{OPTIONS};
  $r = rvals [OPTIONAL TYPE],$nx,$ny,...{OPTIONS};
 
 =for options
@@ -594,8 +559,8 @@ or floating point datatypes.
 For a more general metric, one can define, e.g.,
 
  sub distance {
-   my ($a,$centre,$f) = @_;
-   my ($r) = $a->allaxisvals-$centre;
+   my ($w,$centre,$f) = @_;
+   my ($r) = $w->allaxisvals-$centre;
    $f->($r);
  }
  sub l1 { sumover(abs($_[0])); }
@@ -604,24 +569,22 @@ For a more general metric, one can define, e.g.,
 
 so now
 
- distance($a, $centre, \&euclid);
+ distance($w, $centre, \&euclid);
 
 will emulate rvals, while C<\&l1> and C<\&linfty> will generate other
-well-known norms. 
+well-known norms.
 
 =cut
 
 sub rvals { ref($_[0]) && ref($_[0]) ne 'PDL::Type' ? $_[0]->rvals(@_[1..$#_]) : PDL->rvals(@_) }
 sub PDL::rvals { # Return radial distance from given point and offset
-    my $class = shift;
     my $opt = pop @_ if ref($_[$#_]) eq "HASH";
-    my %opt = defined $opt ? 
+    my %opt = defined $opt ?
                iparse( {
 			CENTRE  => undef, # needed, otherwise centre/center handling painful
 			Squared => 0,
 		       }, $opt ) : ();
-    my $r =  scalar(@_)? $class->new_from_specification(@_) : $class->new_or_inplace;
-
+    my $r = &PDL::Core::_construct;
     my @pos;
     @pos = @{$opt{CENTRE}} if defined $opt{CENTRE};
     my $offset;
@@ -632,7 +595,7 @@ sub PDL::rvals { # Return radial distance from given point and offset
     for ($i=0; $i<$r->getndims; $i++) {
          $offset = (defined $pos[$i] ? $pos[$i] : int($r->getdim($i)/2));
 	 # Note careful coding for speed and min memory footprint
-	 PDL::Primitive::axisvalues($tmp->xchg(0,$i));
+	 PDL::Primitive::axisvalues($tmp->xchg(0,$i)->inplace);
 	 $tmp -= $offset; $tmp *= $tmp;
          $r += $tmp;
     }
@@ -643,19 +606,19 @@ sub PDL::rvals { # Return radial distance from given point and offset
 
 =for ref
 
-Fills a piddle with index values on Nth dimension
+Fills an ndarray with index values on Nth dimension
 
 =for usage
 
- $z = axisvals ($piddle, $nth);
+ $z = axisvals ($ndarray, $nth);
 
-This is the routine, for which L<xvals|/xvals>, L<yvals|/yvals> etc
+This is the routine, for which L</xvals>, L</yvals> etc
 are mere shorthands. C<axisvals> can be used to fill along any dimension,
 using a parameter.
 
-See also L<allaxisvals|allaxisvals>, which generates all axis values 
-simultaneously in a form useful for L<range|range>, L<interpND|interpND>, 
-L<indexND|indexND>, etc.
+See also L</allaxisvals>, which generates all axis values
+simultaneously in a form useful for L</range>, L</interpND>,
+L</indexND>, etc.
 
 Note the 'from specification' style (see L<zeroes|PDL::Core/zeroes>) is
 not available here, for obvious reasons.
@@ -669,25 +632,24 @@ sub PDL::axisvals {
 		# This is 'kind of' consistency...
 		$dummy .= 0;
 		return $dummy;
-#		barf("Too few dimensions given to axisvals $nth\n");
 	}
 	my $bar = $dummy->xchg(0,$nth);
-	PDL::Primitive::axisvalues($bar);
+	PDL::Primitive::axisvalues($bar->inplace);
 	return $dummy;
 }
 
 # We need this version for xvals etc to work in place
 sub axisvals2 {
-	my($this,$nth) = @_;
-	my $dummy = shift;
+	my($dummy,$nth,$keep_type) = @_;
+	$dummy = PDL::Core::double($dummy)
+	  if !$keep_type && $dummy->get_datatype < PDL::Core::double()->enum;
 	if($dummy->getndims() <= $nth) {
 		# This is 'kind of' consistency...
 		$dummy .= 0;
 		return $dummy;
-#		barf("Too few dimensions given to axisvals $nth\n");
 	}
 	my $bar = $dummy->xchg(0,$nth);
-	PDL::Primitive::axisvalues($bar);
+	PDL::Primitive::axisvalues($bar->inplace);
 	return $dummy;
 }
 sub PDL::sec {
@@ -722,52 +684,45 @@ sub PDL::ins {
 }
 
 sub PDL::similar_assign {
-	my($from,$to) = @_;
-	if((join ',',@{$from->dims}) ne (join ',',@{$to->dims})) {
-		barf "Similar_assign: dimensions [".
-			(join ',',@{$from->dims})."] and [".
-			(join ',',@{$to->dims})."] do not match!\n";
-	}
-	$to .= $from;
+  my($from,$to) = @_;
+  if((my $fd = join ',',@{$from->dims}) ne (my $td = join ',',@{$to->dims})) {
+    barf "Similar_assign: dimensions [$fd] and [$td] do not match!\n";
+  }
+  $to .= $from;
 }
 
 =head2 transpose
 
 =for ref
 
-transpose rows and columns. 
+transpose rows and columns.
 
 =for usage
 
- $b = transpose($a); 
+ $y = transpose($w);
 
 =for example
 
- pdl> $a = sequence(3,2)
- pdl> p $a
+ pdl> $w = sequence(3,2)
+ pdl> p $w
  [
   [0 1 2]
   [3 4 5]
- ]                                                                               
- pdl> p transpose( $a )
+ ]
+ pdl> p transpose( $w )
  [
   [0 3]
   [1 4]
-  [2 5]                                                                          
+  [2 5]
  ]
 
 =cut
 
 sub PDL::transpose {
-	my($this) = @_;
-	if($this->getndims <= 1) {
-	    if($this->getndims==0) {
-		return pdl $this->dummy(0)->dummy(0);
-	    } else {
-		return pdl $this->dummy(0);
-	    }
-	}
-	return $this->xchg(0,1);
+  my ($this) = @_;
+  $this->getndims > 1 ? $this->xchg(0,1) :
+  $this->getndims > 0 ? $this->dummy(0) :
+  $this->dummy(0)->dummy(0);
 }
 
 1;
