@@ -1,3 +1,8 @@
+package PDL::LiteF;
+
+use strict;
+use warnings;
+
 =head1 NAME
 
 PDL::LiteF - minimum PDL module function loader
@@ -7,7 +12,7 @@ PDL::LiteF - minimum PDL module function loader
 Loads the smallest possible set of modules for
 PDL to work, making the functions available in
 the current namespace. If you want something even
-smaller see the L<PDL::Lite|PDL::Lite> module.
+smaller see the L<PDL::Lite> module.
 
 =head1 SYNOPSIS
 
@@ -20,23 +25,19 @@ smaller see the L<PDL::Lite|PDL::Lite> module.
    use PDL::Basic;
    use PDL::Slices;
    use PDL::Bad;
-   use PDL::Version;
    use PDL::Lvalue;
 
 =cut
 
-# get the version: 
-use PDL::Version;
-$PDL::LiteF::VERSION = $PDL::Version::VERSION;
+require PDL; # get the version
 
+our $VERSION = $PDL::VERSION;
 
 # Load the fundamental PDL packages, with imports
 
-sub PDL::LiteF::import {
-
-my $pkg = (caller())[0];
-eval <<EOD;
-
+sub import {
+  my $pkg = (caller())[0];
+  eval <<EOD;
 package $pkg;
 
 use PDL::Core;
@@ -49,11 +50,7 @@ use PDL::Bad;
 use PDL::Lvalue;
 
 EOD
-
-die $@ if $@;
-
+  die $@ if $@;
 }
 
-;# Exit with OK status
-
 1;
