@@ -20,7 +20,7 @@ use Carp;
 
 =head1 NAME
 
-PDL::IO::FITS -- Simple FITS support for PDL
+PDL::IO::FITS - Simple FITS support for PDL
 
 =head1 SYNOPSIS
 
@@ -33,9 +33,9 @@ PDL::IO::FITS -- Simple FITS support for PDL
 =head1 DESCRIPTION
 
 This module provides basic FITS support for PDL, in the sense of
-reading and writing whole FITS files.  (For more complex operations,
+reading and writing whole FITS files. For more complex operations,
 such as prefiltering rows out of tables or performing operations on
-the FITS file in-place on disk), you can use the Astro::FITS::CFITSIO
+the FITS file in-place on disk, you can use the Astro::FITS::CFITSIO
 module that is available on CPAN.
 
 Basic FITS image files are supported, along with BINTABLE and IMAGE extensions.
@@ -222,7 +222,7 @@ Each element of the hash contains a PDL (for numerical values) or a
 perl list (for string values).  The PDL's 0th dimension runs across
 rows; the 1st dimension runs across the repeat index within the row
 (for rows with more than one value).  (Note that this is different from
-standard threading order - but it allows Least Surprise to work when
+standard broadcasting order - but it allows Least Surprise to work when
 adding more complicated objects such as collections of numbers (via
 the repeat count) or variable length arrays.)
 
@@ -304,7 +304,7 @@ reading in a data structure as well.
 
 =cut
 
-our $rfits_options = new PDL::Options( { bscale=>1, data=>1, hdrcpy=>0, expand=>1, afh=>1 } );
+our $rfits_options = PDL::Options->new( { bscale=>1, data=>1, hdrcpy=>0, expand=>1, afh=>1 } );
 
 sub PDL::rfitshdr {
   my $class = shift;
@@ -659,9 +659,9 @@ sub treat_bscale($$){
 
     my ($bscale, $bzero);
     $bscale = $$foo{"BSCALE"}; $bzero = $$foo{"BZERO"};
-    print "BSCALE = $bscale &&  BZERO = $bzero\n" if $PDL::verbose;
     $bscale = 1 if (!defined($bscale) || $bscale eq "");
     $bzero  = 0 if (!defined($bzero)  || $bzero  eq "");
+    print "BSCALE = $bscale &&  BZERO = $bzero\n" if $PDL::verbose;
     
     # Be clever and work out the final datatype before eating
     # memory
@@ -1358,7 +1358,7 @@ sub _rfits_unpack_zimage($$$) {
     }
 
     ##########
-    # Slice up the output image plane into tiles, and use the threading engine
+    # Slice up the output image plane into tiles, and use the broadcasting engine
     # to assign everything to them.
     my $cutup = $pdl->range( $tiledex, [@tiledims], 't') # < ntiles, tilesize0..tilesizen >
 	->mv(0,-1)                                       # < tilesize0..tilesizen, ntiles >
@@ -1494,17 +1494,17 @@ dim of the PDL runs across rows, and higher dims are written as
 multi-value entries in the table (e.g. a 7x5 PDL will yield a single
 named column with 7 rows and 5 numerical entries per row, in a binary
 table).  Note that this is slightly different from the usual concept
-of threading, in which dimension 1 runs across rows.
+of broadcasting, in which dimension 1 runs across rows.
 
 ASCII tables only allow one entry per column in each row, so
 if you plan to write an ASCII table then all of the values of C<$hash>
 should have at most one dim.
 
-All of the columns' 0 dims must agree in the threading sense. That is to
+All of the columns' 0 dims must agree in the broadcasting sense. That is to
 say, the 0th dimension of all of the values of C<$hash> should be the
 same (indicating that all columns have the same number of rows).  As
 an exception, if the 0th dim of any of the values is 1, or if that
-value is a PDL scalar (with 0 dims), then that value is "threaded"
+value is a PDL scalar (with 0 dims), then that value is "broadcasted"
 over -- copied into all rows.
 
 Data dimensions higher than 2 are preserved in binary tables,
@@ -1810,7 +1810,7 @@ sub PDL::wfits {
 	  if($all_valid_fits) {
 	      # All the keys look like valid FITS header keywords -- so 
 	      # create a tied FITS header object and use that instead.
-	      my $afh = new Astro::FITS::Header( );
+	      my $afh = Astro::FITS::Header->new;
 	      my %hh;
 	      tie %hh, "Astro::FITS::Header", $afh;
 	      for (keys %$h) {
@@ -2413,7 +2413,7 @@ FOO
 	  $tmp = $$t;
 	} else {                                  # Only other case is ASCII just now...
 	  $tmp = ( ref $x eq 'ARRAY' ) ?          # Switch on array or string
-	    ( $#$x == 0 ? $x->[0] : $x->[$r] )    # Thread arrays as needed
+	    ( $#$x == 0 ? $x->[0] : $x->[$r] )    # broadcast arrays as needed
 	      : $x;
 	 
 	  $tmp .= " " x ($field_len[$c] - length($tmp));
